<?php

declare(strict_types=1);

namespace Drush\Commands\radix;

use Consolidation\AnnotatedCommand\CommandData;
use Consolidation\AnnotatedCommand\CommandError;
use Drush\Commands\DrushCommands;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Process\Process;

/**
 * Class SubThemeCommands.
 */
class SubThemeCommands extends DrushCommands
{
    /**
     * Creates a Radix sub-theme.
     *
     * @command radix:create
     * @aliases radix
     * @param string $name The machine-readable name of your sub-theme.
     * @bootstrap full
     *
     * @usage drush radix:create 'My Theme'
     *   Creates a Radix sub-theme called "My Theme", using the default options.
     * @usage drush radix:create 'My Theme' --machine_name=my_theme
     *   Creates a Radix sub-theme called "My Theme" with a specific machine name.
     */
    public function createSubTheme(string $name) {
        try {
            $this->copyStarterKit();
            $this->generateTheme($name);
            $this->removeCopiedStarterKit();
    
            // Success message
            $this->logger()->success("🚀 Sub-theme '{$name}' created successfully. You may now enable it in the Appearance section of the Drupal administration or by Drush as shown below:");
          
            $this->printHeading(PHP_EOL . "INSTALLATION INSTRUCTIONS");
            $this->logger()->notice("Radix theme uses laravel-mix to compile and bundle SASS and JS." . PHP_EOL);
    
            $this->printHeading("🟡 STEP 1");
            $this->logger()->notice("Make sure you have Node, npm and nvm installed.");
            $this->printCommand('Read the guide on how to install node and npm: https://docs.npmjs.com/downloading-and-installing-node-js-and-npm');
            $this->printCommand('Read the guide on how to install and use nvm: https://github.com/nvm-sh/nvm');
    
            $this->printHeading("🟡 STEP 2");
            $this->logger()->notice("Go to the root of the {$name} theme and run:");
            $this->printCommand('nvm use');

            $this->logger()->notice("then install the required node packages:");
            $this->printCommand('npm install');
    
            $this->printHeading("🟡 STEP 3");
            $this->logger()->notice("Create a copy of .env.example file and rename it to .env.local:");
            $this->printCommand('cp .env.example .env.local');

            $this->logger()->notice("Update DRUPAL_BASE_URL in newly created .env.local file to match your local environment URL.");
            
            $this->printHeading("🟡 STEP 4");
            $this->logger()->notice("Enable and set {$name} as the default theme:");
            $this->printCommand("drush then {$name} -y");
            $this->printCommand("drush config-set system.theme default {$name} -y");

            $this->printHeading("✅ STEP 5");
            $this->logger()->notice("Run the following command to compile Sass, JS and watch for other changes:");
            $this->printCommand('npm run watch');
        } catch (\Exception $exception) {
            $this->logger()->error($exception->getMessage());
        }
    }
    
    private function printCommand(string $command) {
        $formattedCommand = "<fg=green>$command</>";
        $this->output()->writeln($formattedCommand);
    }
    
    private function printHeading(string $heading) {
        $formattedHeading = PHP_EOL . "<options=bold>$heading:</>";
        $this->output()->writeln($formattedHeading);
    }

    private function copyStarterKit() {
        $filesystem = new Filesystem();
        $drupalRoot = \Drush\Drush::bootstrapManager()->getRoot();
        $source = $drupalRoot . '/themes/contrib/radix/src/kits/radix_starterkit';
        $destination = $drupalRoot . '/themes/custom/radix_starterkit';
        $filesystem->mirror($source, $destination);
    }
    
    private function generateTheme(string $themeName) {
        $drupalRoot = \Drush\Drush::bootstrapManager()->getRoot();
        $process = new Process([
            'php', $drupalRoot . '/core/scripts/drupal', 'generate-theme', '--starterkit', 'radix_starterkit', $themeName, '--path', 'themes/custom',
        ]);
        $process->run();
    
        if (!$process->isSuccessful()) {
            throw new \RuntimeException($process->getErrorOutput());
        }
    }

    private function removeCopiedStarterKit() {
        $filesystem = new Filesystem();
        $filesystem->remove('../web/themes/custom/radix_starterkit');
    }

    /**
     * @hook validate radix:create
     */
    public function validateCreateSubTheme(CommandData $commandData): ?CommandError {
        $name = $commandData->input()->getArgument('name');
        if (!$this->isValidName($name)) {
            return new CommandError("Invalid theme name: '$name'. Name must be a non-empty string.");
        }
        return null;
    }

    private function isValidName(string $name): bool{
        return !empty($name);
    }
}
